@@ -42,14 +42,7 @@
 #include "entcode.h"
 #include "os_support.h"
 
-#ifndef OVERRIDE_CELT_ILOG2
-/** Integer log in base2. Undefined for zero and negative numbers */
-static inline celt_int16 celt_ilog2(celt_word32 x)
-{
-   celt_assert2(x>0, "celt_ilog2() only defined for strictly positive numbers");
-   return EC_ILOG(x)-1;
-}
-#endif
+
 
 #ifndef OVERRIDE_FIND_MAX16
 static inline int find_max16(celt_word16 *x, int len)
@@ -85,7 +78,11 @@ static inline int find_max32(celt_word32 *x, int len)
 }
 #endif
 
+/* Multiplies two 16-bit fractional values. Bit-exactness of this macro is important */
 #define FRAC_MUL16(a,b) ((16384+((celt_int32)(celt_int16)(a)*(celt_int16)(b)))>>15)
+
+/* This is a cos() approximation designed to be bit-exact on any platform. Bit exactness
+   with this approximation is important because it has an impact on the bit allocation */
 static inline celt_int16 bitexact_cos(celt_int16 x)
 {
    celt_int32 tmp;
@@ -106,12 +103,14 @@ static inline celt_int16 bitexact_cos(celt_int16 x)
 #define celt_sqrt(x) ((float)sqrt(x))
 #define celt_psqrt(x) ((float)sqrt(x))
 #define celt_rsqrt(x) (1.f/celt_sqrt(x))
+#define celt_rsqrt_norm(x) (celt_rsqrt(x))
 #define celt_acos acos
 #define celt_exp exp
 #define celt_cos_norm(x) (cos((.5f*M_PI)*(x)))
 #define celt_atan atan
 #define celt_rcp(x) (1.f/(x))
 #define celt_div(a,b) ((a)/(b))
+#define frac_div32(a,b) ((float)(a)/(b))
 
 #ifdef FLOAT_APPROX
 
@@ -169,6 +168,16 @@ static inline float celt_exp2(float x)
 
 #include "os_support.h"
 
+#ifndef OVERRIDE_CELT_ILOG2
+/** Integer log in base2. Undefined for zero and negative numbers */
+static inline celt_int16 celt_ilog2(celt_int32 x)
+{
+   celt_assert2(x>0, "celt_ilog2() only defined for strictly positive numbers");
+   return EC_ILOG(x)-1;
+}
+#endif
+
+
 #ifndef OVERRIDE_CELT_MAXABS16
 static inline celt_word16 celt_maxabs16(celt_word16 *x, int len)
 {
@@ -186,17 +195,13 @@ static inline celt_int16 celt_zlog2(celt_word32 x)
    return x <= 0 ? 0 : celt_ilog2(x);
 }
 
-/** Reciprocal sqrt approximation (Q30 input, Q0 output or equivalent) */
-static inline celt_word32 celt_rsqrt(celt_word32 x)
+/** Reciprocal sqrt approximation in the range [0.25,1) (Q16 in, Q14 out) */
+static inline celt_word16 celt_rsqrt_norm(celt_word32 x)
 {
-   int k;
    celt_word16 n;
    celt_word16 r;
    celt_word16 r2;
    celt_word16 y;
-   celt_word32 rt;
-   k = celt_ilog2(x)>>1;
-   x = VSHR32(x, (k-7)<<1);
    /* Range of n is [-16384,32767] ([-0.5,1) in Q15). */
    n = x-32768;
    /* Get a rough initial guess for the root.
@@ -210,15 +215,21 @@ static inline celt_word32 celt_rsqrt(celt_word32 x)
       Range of y is [-1564,1594]. */
    r2 = MULT16_16_Q15(r, r);
    y = SHL16(SUB16(ADD16(MULT16_16_Q15(r2, n), r2), 16384), 1);
-   /* Apply a 2nd-order Householder iteration: r += r*y*(y*0.375-0.5). */
-   rt = ADD16(r, MULT16_16_Q15(r, MULT16_16_Q15(y,
-              SUB16(MULT16_16_Q15(y, 12288), 16384))));
-   /* rt is now the Q14 reciprocal square root of the Q16 x, with a maximum
+   /* Apply a 2nd-order Householder iteration: r += r*y*(y*0.375-0.5).
+      This yields the Q14 reciprocal square root of the Q16 x, with a maximum
        relative error of 1.04956E-4, a (relative) RMSE of 2.80979E-5, and a
        peak absolute error of 2.26591/16384. */
-   /* Most of the error in this function comes from the following shift: */
-   rt = PSHR32(rt,k);
-   return rt;
+   return ADD16(r, MULT16_16_Q15(r, MULT16_16_Q15(y,
+              SUB16(MULT16_16_Q15(y, 12288), 16384))));
+}
+
+/** Reciprocal sqrt approximation (Q30 input, Q0 output or equivalent) */
+static inline celt_word32 celt_rsqrt(celt_word32 x)
+{
+   int k;
+   k = celt_ilog2(x)>>1;
+   x = VSHR32(x, (k-7)<<1);
+   return PSHR32(celt_rsqrt_norm(x), k);
 }
 
 /** Sqrt approximation (QX input, QX/2 output) */
@@ -227,7 +238,7 @@ static inline celt_word32 celt_sqrt(celt_word32 x)
    int k;
    celt_word16 n;
    celt_word32 rt;
-   const celt_word16 C[5] = {23175, 11561, -3011, 1699, -664};
+   static const celt_word16 C[5] = {23175, 11561, -3011, 1699, -664};
    if (x==0)
       return 0;
    k = (celt_ilog2(x)>>1)-7;
@@ -246,7 +257,7 @@ static inline celt_word32 celt_psqrt(celt_word32 x)
    int k;
    celt_word16 n;
    celt_word32 rt;
-   const celt_word16 C[5] = {23175, 11561, -3011, 1699, -664};
+   static const celt_word16 C[5] = {23175, 11561, -3011, 1699, -664};
    k = (celt_ilog2(x)>>1)-7;
    x = VSHR32(x, (k<<1));
    n = x-32768;
@@ -302,16 +313,15 @@ static inline celt_word16 celt_log2(celt_word32 x)
 {
    int i;
    celt_word16 n, frac;
-   /*-0.41446   0.96093  -0.33981   0.15600 */
-   /* -0.4144541824871411+32/16384, 0.9590923197873218, -0.3395129038105771,
-       0.16541096501128538 */
-   const celt_word16 C[4] = {-6758, 15715, -5563, 2708};
+   /* -0.41509302963303146, 0.9609890551383969, -0.31836011537636605,
+       0.15530808010959576, -0.08556153059057618 */
+   static const celt_word16 C[5] = {-6801+(1<<13-DB_SHIFT), 15746, -5217, 2545, -1401};
    if (x==0)
       return -32767;
    i = celt_ilog2(x);
    n = VSHR32(x,i-15)-32768-16384;
-   frac = ADD16(C[0], MULT16_16_Q15(n, ADD16(C[1], MULT16_16_Q15(n, ADD16(C[2], MULT16_16_Q15(n, (C[3])))))));
-   return SHL16(i-13,8)+SHR16(frac,14-8);
+   frac = ADD16(C[0], MULT16_16_Q15(n, ADD16(C[1], MULT16_16_Q15(n, ADD16(C[2], MULT16_16_Q15(n, ADD16(C[3], MULT16_16_Q15(n, C[4]))))))));
+   return SHL16(i-13,DB_SHIFT)+SHR16(frac,14-DB_SHIFT);
 }
 
 /*
@@ -370,12 +380,29 @@ static inline celt_word32 celt_rcp(celt_word32 x)
 
 #define celt_div(a,b) MULT32_32_Q31((celt_word32)(a),celt_rcp(b))
 
+static inline celt_word32 frac_div32(celt_word32 a, celt_word32 b)
+{
+   celt_word16 rcp;
+   celt_word32 result, rem;
+   int shift = 30-celt_ilog2(b);
+   a = SHL32(a,shift);
+   b = SHL32(b,shift);
+
+   /* 16-bit reciprocal */
+   rcp = ROUND16(celt_rcp(ROUND16(b,16)),2);
+   result = SHL32(MULT16_32_Q15(rcp, a),1);
+   rem = a-MULT32_32_Q31(result, b);
+   result += SHL32(MULT16_32_Q15(rcp, rem),1);
+   return result;
+}
 
 #define M1 32767
 #define M2 -21
 #define M3 -11943
 #define M4 4936
 
+/* Atan approximation using a 4th order polynomial. Input is in Q15 format
+   and normalized by pi/4. Output is in Q15 format */
 static inline celt_word16 celt_atan01(celt_word16 x)
 {
    return MULT16_16_P15(x, ADD32(M1, MULT16_16_P15(x, ADD32(M2, MULT16_16_P15(x, ADD32(M3, MULT16_16_P15(M4, x)))))));
@@ -386,6 +413,7 @@ static inline celt_word16 celt_atan01(celt_word16 x)
 #undef M3
 #undef M4
 
+/* atan2() approximation valid for positive input values */
 static inline celt_word16 celt_atan2p(celt_word16 y, celt_word16 x)
 {
    if (y < x)
