@@ -61,9 +61,6 @@ int celt_mode_info(const CELTMode *mode, int request, celt_int32 *value)
       return CELT_INVALID_MODE;
    switch (request)
    {
-      case CELT_GET_FRAME_SIZE:
-         *value = mode->mdctSize;
-         break;
       case CELT_GET_LOOKAHEAD:
          *value = mode->overlap;
          break;
@@ -95,39 +92,42 @@ static const celt_int16 bark_freq[BARK_BANDS+1] = {
 /* This allocation table is per critical band. When creating a mode, the bits get added together 
    into the codec bands, which are sometimes larger than one critical band at low frequency */
 
-#ifdef STDIN_TUNING
-int BITALLOC_SIZE;
-int *band_allocation;
-#else
 #define BITALLOC_SIZE 12
-static const int band_allocation[BARK_BANDS*BITALLOC_SIZE] = 
-   /* 0 100 200 300 400 510 630 770 920 1k  1.2 1.5 1.7 2k  2.3 2.7 3.1 3.7 4.4 5.3 6.4 7.7 9.5 12k 15k  */
-   {  4,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /*0*/
-      2,  2,  1,  1,  2,  2,  1,  1,  1,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, /*1*/
-      2,  2,  2,  1,  2,  2,  2,  2,  2,  2,  2,  2,  4,  5,  7,  7,  7,  5,  4,  0,  0,  0,  0,  0,  0, /*2*/
-      2,  2,  2,  2,  3,  3,  3,  3,  3,  3,  3,  3,  5,  6,  8,  8,  8,  6,  5,  4,  0,  0,  0,  0,  0, /*3*/
-      3,  2,  2,  2,  3,  4,  4,  4,  4,  4,  4,  4,  6,  7,  9,  9,  9,  7,  6,  5,  5,  5,  0,  0,  0, /*4*/
-      3,  3,  3,  4,  4,  5,  6,  6,  6,  6,  6,  7,  7,  9, 10, 10, 10,  9,  6,  5,  5,  5,  5,  1,  0, /*5*/
-      4,  3,  3,  4,  6,  7,  7,  7,  7,  7,  8,  9,  9,  9, 11, 10, 10,  9,  9,  8, 11, 10, 10,  1,  0, /*6*/
-      5,  5,  5,  6,  7,  7,  7,  7,  8,  8,  9, 10, 10, 12, 12, 11, 11, 17, 12, 15, 15, 20, 18, 10,  1, /*7*/
-      6,  7,  7,  7,  8,  8,  8,  8,  9, 10, 11, 12, 14, 17, 18, 21, 22, 27, 29, 39, 37, 38, 40, 35,  1, /*8*/
-      7,  7,  7,  8,  8,  8, 10, 10, 10, 13, 14, 18, 20, 24, 28, 32, 32, 35, 38, 38, 42, 50, 59, 54, 31, /*9*/
-      8,  8,  8,  8,  8,  9, 10, 12, 14, 20, 22, 25, 28, 30, 35, 42, 46, 50, 55, 60, 62, 62, 72, 82, 62, /*10*/
-      9,  9,  9, 10, 12, 13, 15, 18, 22, 30, 32, 35, 40, 45, 55, 62, 66, 70, 85, 90, 92, 92, 92,102, 92, /*11*/
-   };
-#endif
 
-static celt_int16 *compute_ebands(celt_int32 Fs, int frame_size, int nbShortMdcts, int *nbEBands)
+static const celt_int16 eband5ms[] = {
+       0,  1,  2,  3,  4,  5,  6,  7,  8, 10, 12, 14, 16, 20, 24, 28, 34, 40, 48, 60, 78, 100
+};
+
+static const unsigned char band_allocation[] = {
+    /* 0 200 400 600 800  1k 1.2 1.4 1.6  2k 2.4 2.8 3.2  4k 4.8 5.6 6.8  8k 9.6 12k 15.6 */
+      10,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+      10,  3,  8,  2,  0,  0,  0,  0,  1,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
+      10,  6,  8,  6,  5,  4,  3,  2,  7, 10, 11,  9,  7,  3,  1,  0,  0,  0,  0,  0,  0,
+      10, 10, 14, 11, 10,  8,  6,  5, 10, 12, 13, 11,  8,  4,  2,  1,  0,  0,  0,  0,  0,
+      13, 10, 17, 16, 14, 12, 10,  8, 12, 14, 14, 12,  9,  5,  3,  2,  2,  1,  0,  0,  0,
+      17, 21, 23, 26, 24, 20, 17, 16, 17, 18, 16, 14, 11,  6,  3,  2,  2,  1,  1,  0,  0,
+      21, 21, 36, 32, 28, 24, 23, 23, 22, 18, 18, 14, 11,  7,  5,  5,  5,  3,  3,  0,  0,
+      31, 35, 40, 32, 30, 28, 26, 26, 25, 24, 19, 15, 15, 13,  9,  9,  8,  7,  5,  2,  0,
+      42, 46, 46, 37, 35, 34, 33, 32, 34, 35, 32, 31, 27, 24, 23, 23, 18, 14, 11,  7,  0,
+      46, 49, 46, 46, 42, 43, 44, 47, 50, 52, 51, 48, 39, 32, 27, 24, 22, 19, 17, 11,  5,
+      53, 53, 49, 48, 55, 66, 71, 71, 71, 65, 64, 64, 56, 47, 41, 37, 31, 24, 20, 16, 10,
+      60, 64, 74, 74, 87,103,106,102,101,100,101, 95, 80, 69, 63, 55, 47, 36, 26, 21, 15,
+};
+
+static celt_int16 *compute_ebands(celt_int32 Fs, int frame_size, int res, int *nbEBands)
 {
-   int min_bins = 3;
    celt_int16 *eBands;
-   int i, res, min_width, lin, low, high, nBark, offset=0;
-
-   /*if (min_bins < nbShortMdcts)
-      min_bins = nbShortMdcts;*/
-   res = (Fs+frame_size)/(2*frame_size);
-   min_width = min_bins*res;
+   int i, lin, low, high, nBark, offset=0;
 
+   if (Fs == 400*(celt_int32)frame_size && Fs >= 40000)
+   {
+      *nbEBands = sizeof(eband5ms)/sizeof(eband5ms[0])-1;
+      eBands = celt_alloc(sizeof(celt_int16)*(*nbEBands+2));
+      for (i=0;i<*nbEBands+2;i++)
+         eBands[i] = eband5ms[i];
+      eBands[*nbEBands+1] = frame_size;
+      return eBands;
+   }
    /* Find the number of critical bands supported by our sampling rate */
    for (nBark=1;nBark<BARK_BANDS;nBark++)
     if (bark_freq[nBark+1]*2 >= Fs)
@@ -135,10 +135,10 @@ static celt_int16 *compute_ebands(celt_int32 Fs, int frame_size, int nbShortMdct
 
    /* Find where the linear part ends (i.e. where the spacing is more than min_width */
    for (lin=0;lin<nBark;lin++)
-      if (bark_freq[lin+1]-bark_freq[lin] >= min_width)
+      if (bark_freq[lin+1]-bark_freq[lin] >= res)
          break;
-   
-   low = ((bark_freq[lin]/res)+(min_bins-1))/min_bins;
+
+   low = (bark_freq[lin]+res/2)/res;
    high = nBark-lin;
    *nbEBands = low+high;
    eBands = celt_alloc(sizeof(celt_int16)*(*nbEBands+2));
@@ -148,18 +148,20 @@ static celt_int16 *compute_ebands(celt_int32 Fs, int frame_size, int nbShortMdct
    
    /* Linear spacing (min_width) */
    for (i=0;i<low;i++)
-      eBands[i] = min_bins*i;
+      eBands[i] = i;
+   if (low>0)
+      offset = eBands[low-1]*res - bark_freq[lin-1];
    /* Spacing follows critical bands */
    for (i=0;i<high;i++)
    {
       int target = bark_freq[lin+i];
-      eBands[i+low] = (2*target+offset+res)/(2*res);
+      eBands[i+low] = (target+(offset+res)/2)/res;
       offset = eBands[i+low]*res - target;
    }
    /* Enforce the minimum spacing at the boundary */
    for (i=0;i<*nbEBands;i++)
-      if (eBands[i] < min_bins*i)
-         eBands[i] = min_bins*i;
+      if (eBands[i] < i)
+         eBands[i] = i;
    eBands[*nbEBands] = (bark_freq[nBark]+res/2)/res;
    eBands[*nbEBands+1] = frame_size;
    if (eBands[*nbEBands] > eBands[*nbEBands+1])
@@ -168,10 +170,10 @@ static celt_int16 *compute_ebands(celt_int32 Fs, int frame_size, int nbShortMdct
    {
       if (eBands[i+1]-eBands[i] < eBands[i]-eBands[i-1])
       {
-         eBands[i] -= (2*eBands[i]-eBands[i-1]-eBands[i+1]+1)/2;
+         eBands[i] -= (2*eBands[i]-eBands[i-1]-eBands[i+1])/2;
       }
    }
-   /*for (i=0;i<*nbEBands+1;i++)
+   /*for (i=0;i<=*nbEBands+1;i++)
       printf ("%d ", eBands[i]);
    printf ("\n");
    exit(1);*/
@@ -181,45 +183,76 @@ static celt_int16 *compute_ebands(celt_int32 Fs, int frame_size, int nbShortMdct
 
 static void compute_allocation_table(CELTMode *mode, int res)
 {
-   int i, j, nBark;
-   celt_int16 *allocVectors;
-
-   /* Find the number of critical bands supported by our sampling rate */
-   for (nBark=1;nBark<BARK_BANDS;nBark++)
-    if (bark_freq[nBark+1]*2 >= mode->Fs)
-       break;
+   int i, j;
+   unsigned char *allocVectors;
+   int maxBands = sizeof(eband5ms)/sizeof(eband5ms[0])-1;
 
    mode->nbAllocVectors = BITALLOC_SIZE;
-   allocVectors = celt_alloc(sizeof(celt_int16)*(BITALLOC_SIZE*mode->nbEBands));
+   allocVectors = celt_alloc(sizeof(unsigned char)*(BITALLOC_SIZE*mode->nbEBands));
    if (allocVectors==NULL)
       return;
+
+   /* Check for standard mode */
+   if (mode->Fs == 400*(celt_int32)mode->shortMdctSize && mode->Fs >= 40000)
+   {
+      for (i=0;i<BITALLOC_SIZE*mode->nbEBands;i++)
+         allocVectors[i] = band_allocation[i];
+      mode->allocVectors = allocVectors;
+      return;
+   }
+   /* If not the standard mode, interpolate */
+
    /* Compute per-codec-band allocation from per-critical-band matrix */
    for (i=0;i<BITALLOC_SIZE;i++)
    {
       celt_int32 current = 0;
       int eband = 0;
-      for (j=0;j<nBark;j++)
+      /* We may be looping over too many bands, but eband will stop being
+         incremented once we reach the last band */
+      for (j=0;j<maxBands;j++)
       {
-         int edge, low;
+         int edge, low, high;
          celt_int32 alloc;
+         alloc = band_allocation[i*maxBands + j]*(mode->eBands[eband+1]-mode->eBands[eband])<<4;
+         low = eband5ms[j]*200;
+         high = eband5ms[j+1]*200;
          edge = mode->eBands[eband+1]*res;
-         alloc = mode->mdctSize*band_allocation[i*BARK_BANDS+j];
-         if (edge < bark_freq[j+1])
+         while (edge <= high && eband < mode->nbEBands)
          {
-            int num, den;
-            num = alloc * (edge-bark_freq[j]);
-            den = bark_freq[j+1]-bark_freq[j];
-            low = (num+den/2)/den;
-            allocVectors[i*mode->nbEBands+eband] = (current+low+128)/256;
-            current=0;
+            celt_int32 num;
+            int den, bits;
+            int N = (mode->eBands[eband+1]-mode->eBands[eband]);
+            num = alloc * (edge-low);
+            den = high-low;
+            /* Divide with rounding */
+            bits = (2*num+den)/(2*den);
+            allocVectors[i*mode->nbEBands+eband] = (2*(current+bits)+(N<<4))/(2*N<<4);
+            /* Remove the part of the band we just allocated */
+            low = edge;
+            alloc -= bits;
+
+            /* Move to next eband */
+            current = 0;
             eband++;
-            current += alloc-low;
-         } else {
-            current += alloc;
-         }   
+            edge = mode->eBands[eband+1]*res;
+         }
+         current += alloc;
+      }
+      if (eband < mode->nbEBands)
+      {
+         int N = (mode->eBands[eband+1]-mode->eBands[eband]);
+         allocVectors[i*mode->nbEBands+eband] = (2*current+(N<<4))/(2*N<<4);
       }
-      allocVectors[i*mode->nbEBands+eband] = (current+128)/256;
    }
+   /*printf ("\n");
+   for (i=0;i<BITALLOC_SIZE;i++)
+   {
+      for (j=0;j<mode->nbEBands;j++)
+         printf ("%d ", allocVectors[i*mode->nbEBands+j]);
+      printf ("\n");
+   }
+   exit(0);*/
+
    mode->allocVectors = allocVectors;
 }
 
@@ -228,6 +261,7 @@ static void compute_allocation_table(CELTMode *mode, int res)
 CELTMode *celt_mode_create(celt_int32 Fs, int frame_size, int *error)
 {
    int i;
+   int LM;
 #ifdef STDIN_TUNING
    scanf("%d ", &MIN_BINS);
    scanf("%d ", &BITALLOC_SIZE);
@@ -243,15 +277,12 @@ CELTMode *celt_mode_create(celt_int32 Fs, int frame_size, int *error)
    ALLOC_STACK;
 #if !defined(VAR_ARRAYS) && !defined(USE_ALLOCA)
    if (global_stack==NULL)
-   {
-      celt_free(global_stack);
       goto failure;
-   }
 #endif 
    for (i=0;i<TOTAL_MODES;i++)
    {
       if (Fs == static_mode_list[i]->Fs &&
-          frame_size == static_mode_list[i]->mdctSize)
+          frame_size == static_mode_list[i]->shortMdctSize*static_mode_list[i]->nbShortMdcts)
       {
          m = static_mode_list[i];
          break;
@@ -268,18 +299,17 @@ CELTMode *celt_mode_create(celt_int32 Fs, int frame_size, int *error)
    if (mode==NULL)
       goto failure;
    CELT_COPY(mode, m, 1);
+   mode->bits = mode->_bits+1;
    mode->marker_start = MODEPARTIAL;
 #else
    int res;
    CELTMode *mode=NULL;
    celt_word16 *window;
+   celt_int16 *logN;
    ALLOC_STACK;
 #if !defined(VAR_ARRAYS) && !defined(USE_ALLOCA)
    if (global_stack==NULL)
-   {
-      celt_free(global_stack);
       goto failure;
-   }
 #endif 
 
    /* The good thing here is that permutation of the arguments will automatically be invalid */
@@ -298,54 +328,46 @@ CELTMode *celt_mode_create(celt_int32 Fs, int frame_size, int *error)
          *error = CELT_BAD_ARG;
       return NULL;
    }
-   res = (Fs+frame_size)/(2*frame_size);
    
    mode = celt_alloc(sizeof(CELTMode));
    if (mode==NULL)
       goto failure;
    mode->marker_start = MODEPARTIAL;
    mode->Fs = Fs;
-   mode->mdctSize = frame_size;
    mode->ePredCoef = QCONST16(.8f,15);
 
-   if (frame_size > 640 && (frame_size%16)==0)
+   if (frame_size >= 640 && (frame_size%16)==0)
    {
-     mode->nbShortMdcts = 8;
-   } else if (frame_size > 384 && (frame_size%8)==0)
+     LM = 3;
+   } else if (frame_size >= 320 && (frame_size%8)==0)
    {
-     mode->nbShortMdcts = 4;
-   } else if (frame_size > 384 && (frame_size%10)==0)
+     LM = 2;
+   } else if (frame_size >= 160 && (frame_size%4)==0)
    {
-     mode->nbShortMdcts = 5;
-   } else if (frame_size > 256 && (frame_size%6)==0)
-   {
-     mode->nbShortMdcts = 3;
-   } else if (frame_size > 256 && (frame_size%8)==0)
-   {
-     mode->nbShortMdcts = 4;
-   } else if (frame_size > 64 && (frame_size%4)==0)
-   {
-     mode->nbShortMdcts = 2;
-   } else if (frame_size > 128 && (frame_size%6)==0)
-   {
-     mode->nbShortMdcts = 3;
+     LM = 1;
    } else
    {
-     mode->nbShortMdcts = 1;
+     LM = 0;
    }
 
-   mode->eBands = compute_ebands(Fs, frame_size, mode->nbShortMdcts, &mode->nbEBands);
+   mode->maxLM = LM;
+   mode->nbShortMdcts = 1<<LM;
+   mode->shortMdctSize = frame_size/mode->nbShortMdcts;
+   res = (mode->Fs+mode->shortMdctSize)/(2*mode->shortMdctSize);
+
+   mode->eBands = compute_ebands(Fs, mode->shortMdctSize, res, &mode->nbEBands);
    if (mode->eBands==NULL)
       goto failure;
 
-   mode->pitchEnd = 4000*(celt_int32)frame_size/Fs;
+   mode->pitchEnd = 4000*(celt_int32)mode->shortMdctSize/Fs;
    
    /* Overlap must be divisible by 4 */
    if (mode->nbShortMdcts > 1)
-      mode->overlap = ((frame_size/mode->nbShortMdcts)>>2)<<2; 
+      mode->overlap = (mode->shortMdctSize>>2)<<2;
    else
       mode->overlap = (frame_size>>3)<<2;
 
+
    compute_allocation_table(mode, res);
    if (mode->allocVectors==NULL)
       goto failure;
@@ -359,34 +381,40 @@ CELTMode *celt_mode_create(celt_int32 Fs, int frame_size, int *error)
       window[i] = Q15ONE*sin(.5*M_PI* sin(.5*M_PI*(i+.5)/mode->overlap) * sin(.5*M_PI*(i+.5)/mode->overlap));
 #else
    for (i=0;i<mode->overlap;i++)
-      window[i] = MIN32(32767,32768.*sin(.5*M_PI* sin(.5*M_PI*(i+.5)/mode->overlap) * sin(.5*M_PI*(i+.5)/mode->overlap)));
+      window[i] = MIN32(32767,floor(.5+32768.*sin(.5*M_PI* sin(.5*M_PI*(i+.5)/mode->overlap) * sin(.5*M_PI*(i+.5)/mode->overlap))));
 #endif
    mode->window = window;
 
-   mode->bits = (const celt_int16 **)compute_alloc_cache(mode, 1);
-   if (mode->bits==NULL)
+   mode->bits = mode->_bits+1;
+   for (i=0;(1<<i)<=mode->nbShortMdcts;i++)
+   {
+      mode->bits[i] = (const celt_int16 **)compute_alloc_cache(mode, 1<<i);
+      if (mode->bits[i]==NULL)
+         goto failure;
+   }
+   mode->bits[-1] = (const celt_int16 **)compute_alloc_cache(mode, 0);
+   if (mode->bits[-1]==NULL)
       goto failure;
 
-#ifndef SHORTCUTS
-   psydecay_init(&mode->psy, MAX_PERIOD/2, mode->Fs);
-   if (mode->psy.decayR==NULL)
+   logN = (celt_int16*)celt_alloc(mode->nbEBands*sizeof(celt_int16));
+   if (logN==NULL)
       goto failure;
-#endif
-   
+
+   for (i=0;i<mode->nbEBands;i++)
+      logN[i] = log2_frac(mode->eBands[i+1]-mode->eBands[i], BITRES);
+   mode->logN = logN;
 #endif /* !STATIC_MODES */
 
-   mdct_init(&mode->mdct, 2*mode->mdctSize);
-   mode->fft = pitch_state_alloc(MAX_PERIOD);
-
-   mode->shortMdctSize = mode->mdctSize/mode->nbShortMdcts;
-   mdct_init(&mode->shortMdct, 2*mode->shortMdctSize);
-   mode->shortWindow = mode->window;
-   mode->prob = quant_prob_alloc(mode);
-   if ((mode->mdct.trig==NULL) || (mode->shortMdct.trig==NULL)
+   clt_mdct_init(&mode->mdct, 2*mode->shortMdctSize*mode->nbShortMdcts, LM);
+   if ((mode->mdct.trig==NULL)
 #ifndef ENABLE_TI_DSPLIB55
-        || (mode->mdct.kfft==NULL) || (mode->fft==NULL) || (mode->shortMdct.kfft==NULL)
+         || (mode->mdct.kfft==NULL)
 #endif
-        || (mode->prob==NULL))
+   )
+      goto failure;
+
+   mode->prob = quant_prob_alloc(mode);
+   if (mode->prob==NULL)
      goto failure;
 
    mode->marker_start = MODEVALID;
@@ -404,7 +432,7 @@ failure:
 
 void celt_mode_destroy(CELTMode *mode)
 {
-   int i;
+   int i, m;
    const celt_int16 *prevPtr = NULL;
    if (mode == NULL)
    {
@@ -425,30 +453,43 @@ void celt_mode_destroy(CELTMode *mode)
    }
    mode->marker_start = MODEFREED;
 #ifndef STATIC_MODES
-   if (mode->bits!=NULL)
+   for (m=0;(1<<m)<=mode->nbShortMdcts;m++)
+   {
+      if (mode->bits[m]!=NULL)
+      {
+         for (i=0;i<mode->nbEBands;i++)
+         {
+            if (mode->bits[m][i] != prevPtr)
+            {
+               prevPtr = mode->bits[m][i];
+               celt_free((int*)mode->bits[m][i]);
+            }
+         }
+      }
+      celt_free((celt_int16**)mode->bits[m]);
+   }
+   if (mode->bits[-1]!=NULL)
    {
       for (i=0;i<mode->nbEBands;i++)
       {
-         if (mode->bits[i] != prevPtr)
+         if (mode->bits[-1][i] != prevPtr)
          {
-            prevPtr = mode->bits[i];
-            celt_free((int*)mode->bits[i]);
-          }
+            prevPtr = mode->bits[-1][i];
+            celt_free((int*)mode->bits[-1][i]);
+         }
       }
-   }   
-   celt_free((int**)mode->bits);
-   celt_free((int*)mode->eBands);
-   celt_free((int*)mode->allocVectors);
+   }
+   celt_free((celt_int16**)mode->bits[-1]);
+
+   celt_free((celt_int16*)mode->eBands);
+   celt_free((celt_int16*)mode->allocVectors);
    
    celt_free((celt_word16*)mode->window);
+   celt_free((celt_int16*)mode->logN);
 
-#ifndef SHORTCUTS
-   psydecay_clear(&mode->psy);
 #endif
-#endif
-   mdct_clear(&mode->mdct);
-   mdct_clear(&mode->shortMdct);
-   pitch_state_free(mode->fft);
+   clt_mdct_clear(&mode->mdct);
+
    quant_prob_free(mode->prob);
    mode->marker_end = MODEFREED;
    celt_free((CELTMode *)mode);
