@@ -39,8 +39,12 @@
 #include "arch.h"
 #include "mdct.h"
 #include "pitch.h"
+#include "entenc.h"
+#include "entdec.h"
 
-#define CELT_BITSTREAM_VERSION 0x8000000b
+#define MAX_CONFIG_SIZES 5
+
+#define CELT_BITSTREAM_VERSION 0x8000000c
 
 #ifdef STATIC_MODES
 #include "static_modes.h"
@@ -64,8 +68,6 @@
 # endif
 #endif
 
-#define MDCT(mode) (&(mode)->mdct)
-
 #ifndef OVERLAP
 #define OVERLAP(mode) ((mode)->overlap)
 #endif
@@ -81,7 +83,6 @@ struct CELTMode {
    celt_uint32 marker_start;
    celt_int32 Fs;
    int          overlap;
-   int          mdctSize;
 
    int          nbEBands;
    int          pitchEnd;
@@ -91,24 +92,30 @@ struct CELTMode {
    celt_word16 ePredCoef;/**< Prediction coefficient for the energy encoding */
    
    int          nbAllocVectors; /**< Number of lines in the matrix below */
-   const celt_int16   *allocVectors;   /**< Number of bits in each band for several rates */
+   const unsigned char   *allocVectors;   /**< Number of bits in each band for several rates */
    
-   const celt_int16 * const *bits; /**< Cache for pulses->bits mapping in each band */
+   const celt_int16 * const **bits;
+   const celt_int16 * const *(_bits[MAX_CONFIG_SIZES]); /**< Cache for pulses->bits mapping in each band */
 
    /* Stuff that could go in the {en,de}coder, but we save space this way */
    mdct_lookup mdct;
 
    const celt_word16 *window;
 
+   int         maxLM;
    int         nbShortMdcts;
    int         shortMdctSize;
-   mdct_lookup shortMdct;
-   const celt_word16 *shortWindow;
 
    int *prob;
+   const celt_int16 *logN;
    celt_uint32 marker_end;
 };
 
 int check_mode(const CELTMode *mode);
 
+/* Prototypes for _ec versions of the encoder/decoder calls (not public) */
+int celt_encode_with_ec(CELTEncoder * restrict st, const celt_int16 * pcm, celt_int16 * optional_resynthesis, int frame_size, unsigned char *compressed, int nbCompressedBytes, ec_enc *enc);
+int celt_encode_with_ec_float(CELTEncoder * restrict st, const float * pcm, float * optional_resynthesis, int frame_size, unsigned char *compressed, int nbCompressedBytes, ec_enc *enc);
+int celt_decode_with_ec(CELTDecoder * restrict st, const unsigned char *data, int len, celt_int16 * restrict pcm, int frame_size, ec_dec *dec);
+int celt_decode_with_ec_float(CELTDecoder * restrict st, const unsigned char *data, int len, float * restrict pcm, int frame_size, ec_dec *dec);
 #endif
