@@ -1,5 +1,6 @@
-/* (C) 2007 Jean-Marc Valin, CSIRO
-*/
+/* Copyright (c) 2007-2008 CSIRO
+   Copyright (c) 2007-2009 Xiph.Org Foundation
+   Written by Jean-Marc Valin */
 /*
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
@@ -40,17 +41,20 @@
 #include <math.h>
 #include <string.h>
 
+#define MAX_PACKET 1024
+
 int main(int argc, char *argv[])
 {
+   int err;
    char *inFile, *outFile;
    FILE *fin, *fout;
    CELTMode *mode=NULL;
    CELTEncoder *enc;
    CELTDecoder *dec;
    int len;
-   celt_int32_t frame_size, channels;
+   celt_int32 frame_size, channels;
    int bytes_per_packet;
-   unsigned char data[1024];
+   unsigned char data[MAX_PACKET];
    int rate;
    int complexity;
 #if !(defined (FIXED_POINT) && defined(STATIC_MODES))
@@ -58,19 +62,21 @@ int main(int argc, char *argv[])
    double rmsd = 0;
 #endif
    int count = 0;
-   celt_int32_t skip;
-   celt_int16_t *in, *out;
+   celt_int32 skip;
+   celt_int16 *in, *out;
    if (argc != 9 && argc != 8 && argc != 7)
    {
-      fprintf (stderr, "Usage: testcelt <rate> <channels> <frame size> <bytes per packet> [<complexity> [packet loss rate]] <input> <output>\n");
+      fprintf (stderr, "Usage: testcelt <rate> <channels> <frame size> "
+               " <bytes per packet> [<complexity> [packet loss rate]] "
+               "<input> <output>\n");
       return 1;
    }
    
    rate = atoi(argv[1]);
    channels = atoi(argv[2]);
    frame_size = atoi(argv[3]);
-   mode = celt051_mode_create(rate, channels, frame_size, NULL);
-   celt051_mode_info(mode, CELT_GET_LOOKAHEAD, &skip);
+   mode = celt_mode_create(rate, frame_size, NULL);
+   celt_mode_info(mode, CELT_GET_LOOKAHEAD, &skip);
    
    if (mode == NULL)
    {
@@ -79,9 +85,10 @@ int main(int argc, char *argv[])
    }
    
    bytes_per_packet = atoi(argv[4]);
-   if (bytes_per_packet < 0 || bytes_per_packet > 200)
+   if (bytes_per_packet < 0 || bytes_per_packet > MAX_PACKET)
    {
-      fprintf (stderr, "bytes per packet must be between 10 and 200\n");
+      fprintf (stderr, "bytes per packet must be between 0 and %d\n",
+                        MAX_PACKET);
       return 1;
    }
 
@@ -100,29 +107,36 @@ int main(int argc, char *argv[])
       return 1;
    }
    
-   /* Use mode4 for stereo and don't forget to change the value of CHANNEL above */
-   enc = celt051_encoder_create(mode);
-   dec = celt051_decoder_create(mode);
+   enc = celt_encoder_create(mode, channels, &err);
+   if (err != 0)
+   {
+      fprintf(stderr, "Failed to create the encoder: %s\n", celt_strerror(err));
+      return 1;
+   }
+   dec = celt_decoder_create(mode, channels, &err);
+   if (err != 0)
+   {
+      fprintf(stderr, "Failed to create the decoder: %s\n", celt_strerror(err));
+      return 1;
+   }
 
    if (argc>7)
    {
       complexity=atoi(argv[5]);
-      celt051_encoder_ctl(enc,CELT_SET_COMPLEXITY(complexity));
+      celt_encoder_ctl(enc,CELT_SET_COMPLEXITY(complexity));
    }
    
-   celt051_mode_info(mode, CELT_GET_FRAME_SIZE, &frame_size);
-   celt051_mode_info(mode, CELT_GET_NB_CHANNELS, &channels);
-   in = (celt_int16_t*)malloc(frame_size*channels*sizeof(celt_int16_t));
-   out = (celt_int16_t*)malloc(frame_size*channels*sizeof(celt_int16_t));
+   in = (celt_int16*)malloc(frame_size*channels*sizeof(celt_int16));
+   out = (celt_int16*)malloc(frame_size*channels*sizeof(celt_int16));
    while (!feof(fin))
    {
-      fread(in, sizeof(short), frame_size*channels, fin);
+      err = fread(in, sizeof(short), frame_size*channels, fin);
       if (feof(fin))
          break;
-      len = celt051_encode(enc, in, in, data, bytes_per_packet);
+      len = celt_encode_resynthesis(enc, in, in, frame_size, data, bytes_per_packet);
       if (len <= 0)
       {
-         fprintf (stderr, "celt051_encode() returned %d\n", len);
+         fprintf (stderr, "celt_encode() returned %d\n", len);
          return 1;
       }
       /* This is for simulating bit errors */
@@ -149,11 +163,11 @@ int main(int argc, char *argv[])
 #endif
 #if 1 /* Set to zero to use the encoder's output instead */
       /* This is to simulate packet loss */
-      if (argc==10 && rand()%1000<atoi(argv[argc-3]))
+      if (argc==9 && rand()%1000<atoi(argv[argc-3]))
       /*if (errors && (errors%2==0))*/
-         celt051_decode(dec, NULL, len, out);
+         celt_decode(dec, NULL, len, out, frame_size);
       else
-         celt051_decode(dec, data, len, out);
+         celt_decode(dec, data, len, out, frame_size);
 #else
       for (i=0;i<frame_size*channels;i++)
          out[i] = in[i];
@@ -171,10 +185,13 @@ int main(int argc, char *argv[])
    }
    PRINT_MIPS(stderr);
    
-   celt051_encoder_destroy(enc);
-   celt051_decoder_destroy(dec);
+   celt_encoder_destroy(enc);
+   celt_decoder_destroy(dec);
    fclose(fin);
    fclose(fout);
+   celt_mode_destroy(mode);
+   free(in);
+   free(out);
 #if !(defined (FIXED_POINT) && defined(STATIC_MODES))
    if (rmsd > 0)
    {
@@ -186,9 +203,6 @@ int main(int argc, char *argv[])
       fprintf (stderr, "Encoder matches decoder!!\n");
    }
 #endif
-   celt051_mode_destroy(mode);
-   free(in);
-   free(out);
    return 0;
 }
 
