@@ -1,5 +1,6 @@
-/* (C) 2007-2008 Jean-Marc Valin, CSIRO
-*/
+/* Copyright (c) 2007-2008 CSIRO
+   Copyright (c) 2007-2009 Xiph.Org Foundation
+   Written by Jean-Marc Valin */
 /*
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
@@ -38,124 +39,198 @@
 #include "vq.h"
 #include "arch.h"
 #include "os_support.h"
+#include "rate.h"
+
+#ifndef M_PI
+#define M_PI 3.141592653
+#endif
+
+
+static void exp_rotation1(celt_norm *X, int len, int dir, int stride, celt_word16 c, celt_word16 s)
+{
+   int i;
+   celt_norm *Xptr;
+   if (dir>0)
+      s = -s;
+   Xptr = X;
+   for (i=0;i<len-stride;i++)
+   {
+      celt_norm x1, x2;
+      x1 = Xptr[0];
+      x2 = Xptr[stride];
+      Xptr[stride] = EXTRACT16(SHR32(MULT16_16(c,x2) + MULT16_16(s,x1), 15));
+      *Xptr++      = EXTRACT16(SHR32(MULT16_16(c,x1) - MULT16_16(s,x2), 15));
+   }
+   Xptr = &X[len-2*stride-1];
+   for (i=len-2*stride-1;i>=0;i--)
+   {
+      celt_norm x1, x2;
+      x1 = Xptr[0];
+      x2 = Xptr[stride];
+      Xptr[stride] = EXTRACT16(SHR32(MULT16_16(c,x2) + MULT16_16(s,x1), 15));
+      *Xptr--      = EXTRACT16(SHR32(MULT16_16(c,x1) - MULT16_16(s,x2), 15));
+   }
+}
+
+static void exp_rotation(celt_norm *X, int len, int dir, int stride, int K)
+{
+   int i;
+   celt_word16 c, s;
+   celt_word16 gain, theta;
+   int stride2=0;
+   /*int i;
+   if (len>=30)
+   {
+      for (i=0;i<len;i++)
+         X[i] = 0;
+      X[14] = 1;
+      K=5;
+   }*/
+   if (2*K>=len)
+      return;
+   gain = celt_div((celt_word32)MULT16_16(Q15_ONE,len),(celt_word32)(len+10*K));
+   /* FIXME: Make that HALF16 instead of HALF32 */
+   theta = HALF32(MULT16_16_Q15(gain,gain));
+
+   c = celt_cos_norm(EXTEND32(theta));
+   s = celt_cos_norm(EXTEND32(SUB16(Q15ONE,theta))); /*  sin(theta) */
+
+   if (len>=8*stride)
+   {
+      stride2 = 1;
+      /* This is just a simple way of computing sqrt(len/stride) with rounding.
+         It's basically incrementing long as (stride2+0.5)^2 < len/stride.
+         I _think_ it is bit-exact */
+      while ((stride2*stride2+stride2)*stride + (stride>>2) < len)
+         stride2++;
+   }
+   len /= stride;
+   for (i=0;i<stride;i++)
+   {
+      if (dir < 0)
+      {
+         if (stride2)
+            exp_rotation1(X+i*len, len, dir, stride2, s, c);
+         exp_rotation1(X+i*len, len, dir, 1, c, s);
+      } else {
+         exp_rotation1(X+i*len, len, dir, 1, c, s);
+         if (stride2)
+            exp_rotation1(X+i*len, len, dir, stride2, s, c);
+      }
+   }
+   /*if (len>=30)
+   {
+      for (i=0;i<len;i++)
+         printf ("%f ", X[i]);
+      printf ("\n");
+      exit(0);
+   }*/
+}
 
 /** Takes the pitch vector and the decoded residual vector, computes the gain
     that will give ||p+g*y||=1 and mixes the residual with the pitch. */
-static void mix_pitch_and_residual(int * restrict iy, celt_norm_t * restrict X, int N, int K, const celt_norm_t * restrict P)
+static void normalise_residual(int * restrict iy, celt_norm * restrict X, int N, int K, celt_word32 Ryy)
 {
    int i;
-   celt_word32_t Ryp, Ryy, Rpp;
-   celt_word16_t ryp, ryy, rpp;
-   celt_word32_t g;
-   VARDECL(celt_norm_t, y);
 #ifdef FIXED_POINT
-   int yshift;
+   int k;
 #endif
-   SAVE_STACK;
+   celt_word32 t;
+   celt_word16 g;
+
 #ifdef FIXED_POINT
-   yshift = 13-celt_ilog2(K);
+   k = celt_ilog2(Ryy)>>1;
 #endif
-   ALLOC(y, N, celt_norm_t);
-
-   /*for (i=0;i<N;i++)
-   printf ("%d ", iy[i]);*/
-   Rpp = 0;
-   i=0;
-   do {
-      Rpp = MAC16_16(Rpp,P[i],P[i]);
-      y[i] = SHL16(iy[i],yshift);
-   } while (++i < N);
-
-   Ryp = 0;
-   Ryy = 0;
-   /* If this doesn't generate a dual MAC (on supported archs), fire the compiler guy */
-   i=0;
-   do {
-      Ryp = MAC16_16(Ryp, y[i], P[i]);
-      Ryy = MAC16_16(Ryy, y[i], y[i]);
-   } while (++i < N);
-
-   ryp = ROUND16(Ryp,14);
-   ryy = ROUND16(Ryy,14);
-   rpp = ROUND16(Rpp,14);
-   /* g = (sqrt(Ryp^2 + Ryy - Rpp*Ryy)-Ryp)/Ryy */
-   g = MULT16_32_Q15(celt_sqrt(MAC16_16(Ryy, ryp,ryp) - MULT16_16(ryy,rpp)) - ryp,
-                     celt_rcp(SHR32(Ryy,9)));
+   t = VSHR32(Ryy, (k-7)<<1);
+   g = celt_rsqrt_norm(t);
 
    i=0;
-   do 
-      X[i] = ADD16(P[i], ROUND16(MULT16_16(y[i], g),11));
+   do
+      X[i] = EXTRACT16(PSHR32(MULT16_16(g, iy[i]), k+1));
    while (++i < N);
-
-   RESTORE_STACK;
 }
 
-
-void alg_quant(celt_norm_t *X, celt_mask_t *W, int N, int K, celt_norm_t *P, ec_enc *enc)
+void alg_quant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, int resynth, ec_enc *enc)
 {
-   VARDECL(celt_norm_t, y);
+   VARDECL(celt_norm, y);
    VARDECL(int, iy);
-   VARDECL(celt_word16_t, signx);
+   VARDECL(celt_word16, signx);
    int j, is;
-   celt_word16_t s;
+   celt_word16 s;
    int pulsesLeft;
-   celt_word32_t sum;
-   celt_word32_t xy, yy, yp;
-   celt_word16_t Rpp;
+   celt_word32 sum;
+   celt_word32 xy, yy;
    int N_1; /* Inverse of N, in Q14 format (even for float) */
 #ifdef FIXED_POINT
    int yshift;
 #endif
    SAVE_STACK;
 
+   if (K==0)
+   {
+      if (lowband != NULL && resynth)
+      {
+         for (j=0;j<N;j++)
+            X[j] = lowband[j];
+         renormalise_vector(X, Q15ONE, N, 1);
+      } else {
+         /* This is important for encoding the side in stereo mode */
+         for (j=0;j<N;j++)
+            X[j] = 0;
+      }
+      return;
+   }
+   K = get_pulses(K);
 #ifdef FIXED_POINT
    yshift = 13-celt_ilog2(K);
 #endif
 
-   ALLOC(y, N, celt_norm_t);
+   ALLOC(y, N, celt_norm);
    ALLOC(iy, N, int);
-   ALLOC(signx, N, celt_word16_t);
+   ALLOC(signx, N, celt_word16);
    N_1 = 512/N;
+   
+   if (spread)
+      exp_rotation(X, N, 1, spread, K);
 
    sum = 0;
    j=0; do {
-      X[j] -= P[j];
       if (X[j]>0)
          signx[j]=1;
       else {
          signx[j]=-1;
          X[j]=-X[j];
-         P[j]=-P[j];
       }
       iy[j] = 0;
       y[j] = 0;
-      sum = MAC16_16(sum, P[j],P[j]);
    } while (++j<N);
-   Rpp = ROUND16(sum, NORM_SHIFT);
 
-   celt_assert2(Rpp<=NORM_SCALING, "Rpp should never have a norm greater than unity");
-
-   xy = yy = yp = 0;
+   xy = yy = 0;
 
    pulsesLeft = K;
 
    /* Do a pre-search by projecting on the pyramid */
    if (K > (N>>1))
    {
-      celt_word16_t rcp;
-      sum=0;
+      celt_word16 rcp;
       j=0; do {
          sum += X[j];
       }  while (++j<N);
-      if (sum == 0)
+
+#ifdef FIXED_POINT
+      if (sum <= K)
+#else
+      if (sum <= EPSILON)
+#endif
       {
-         X[0] = 16384;
-         sum = 16384;
+         X[0] = QCONST16(1.f,14);
+         j=1; do
+            X[j]=0;
+         while (++j<N);
+         sum = QCONST16(1.f,14);
       }
       /* Do we have sufficient accuracy here? */
       rcp = EXTRACT16(MULT16_32_Q16(K-1, celt_rcp(sum)));
-      /*rcp = DIV32(SHL32(EXTEND32(K-1),15),EPSILON+sum);*/
-      /*printf ("%d (%d %d)\n", rcp, N, K);*/
       j=0; do {
 #ifdef FIXED_POINT
          /* It's really important to round *towards zero* here */
@@ -166,22 +241,19 @@ void alg_quant(celt_norm_t *X, celt_mask_t *W, int N, int K, celt_norm_t *P, ec_
          y[j] = SHL16(iy[j],yshift);
          yy = MAC16_16(yy, y[j],y[j]);
          xy = MAC16_16(xy, X[j],y[j]);
-         yp += P[j]*y[j];
          y[j] *= 2;
          pulsesLeft -= iy[j];
       }  while (++j<N);
    }
-   /*if (pulsesLeft > N+2)
-      printf ("%d / %d (%d)\n", pulsesLeft, K, N);*/
    celt_assert2(pulsesLeft>=1, "Allocated too many pulses in the quick pass");
 
-   while (pulsesLeft > 1)
+   while (pulsesLeft > 0)
    {
       int pulsesAtOnce=1;
       int best_id;
-      celt_word16_t magnitude;
-      celt_word32_t best_num = -VERY_LARGE16;
-      celt_word16_t best_den = 0;
+      celt_word16 magnitude;
+      celt_word32 best_num = -VERY_LARGE16;
+      celt_word16 best_den = 0;
 #ifdef FIXED_POINT
       int rshift;
 #endif
@@ -202,7 +274,7 @@ void alg_quant(celt_norm_t *X, celt_mask_t *W, int N, int K, celt_norm_t *P, ec_
          /* This should ensure that anything we can process will have a better score */
       j=0;
       do {
-         celt_word16_t Rxy, Ryy;
+         celt_word16 Rxy, Ryy;
          /* Select sign based on X[j] alone */
          s = magnitude;
          /* Temporary sums of the new pulse(s) */
@@ -232,7 +304,6 @@ void alg_quant(celt_norm_t *X, celt_mask_t *W, int N, int K, celt_norm_t *P, ec_
       xy = xy + MULT16_16(s,X[j]);
       /* We're multiplying y[j] by two so we don't have to do it here */
       yy = yy + MULT16_16(s,y[j]);
-      yp = yp + MULT16_16(s, P[j]);
 
       /* Only now that we've made the final choice, update y/iy */
       /* Multiplying y[j] by 2 so we don't have to do it everywhere else */
@@ -240,54 +311,8 @@ void alg_quant(celt_norm_t *X, celt_mask_t *W, int N, int K, celt_norm_t *P, ec_
       iy[j] += is;
       pulsesLeft -= pulsesAtOnce;
    }
-   
-   if (pulsesLeft > 0)
-   {
-      celt_word16_t g;
-      celt_word16_t best_num = -VERY_LARGE16;
-      celt_word16_t best_den = 0;
-      int best_id = 0;
-      celt_word16_t magnitude = SHL16(1, yshift);
-
-      /* The squared magnitude term gets added anyway, so we might as well 
-      add it outside the loop */
-      yy = MAC16_16(yy, magnitude,magnitude);
-      j=0;
-      do {
-         celt_word16_t Rxy, Ryy, Ryp;
-         celt_word16_t num;
-         /* Select sign based on X[j] alone */
-         s = magnitude;
-         /* Temporary sums of the new pulse(s) */
-         Rxy = ROUND16(MAC16_16(xy, s,X[j]), 14);
-         /* We're multiplying y[j] by two so we don't have to do it here */
-         Ryy = ROUND16(MAC16_16(yy, s,y[j]), 14);
-         Ryp = ROUND16(MAC16_16(yp, s,P[j]), 14);
-
-            /* Compute the gain such that ||p + g*y|| = 1 
-         ...but instead, we compute g*Ryy to avoid dividing */
-         g = celt_psqrt(MULT16_16(Ryp,Ryp) + MULT16_16(Ryy,QCONST16(1.f,14)-Rpp)) - Ryp;
-            /* Knowing that gain, what's the error: (x-g*y)^2 
-         (result is negated and we discard x^2 because it's constant) */
-         /* score = 2*g*Rxy - g*g*Ryy;*/
-#ifdef FIXED_POINT
-         /* No need to multiply Rxy by 2 because we did it earlier */
-         num = MULT16_16_Q15(ADD16(SUB16(Rxy,g),Rxy),g);
-#else
-         num = g*(2*Rxy-g);
-#endif
-         if (MULT16_16(best_den, num) > MULT16_16(Ryy, best_num))
-         {
-            best_den = Ryy;
-            best_num = num;
-            best_id = j;
-         }
-      } while (++j<N);
-      iy[best_id] += 1;
-   }
    j=0;
    do {
-      P[j] = MULT16_16(signx[j],P[j]);
       X[j] = MULT16_16(signx[j],X[j]);
       if (signx[j] < 0)
          iy[j] = -iy[j];
@@ -296,74 +321,79 @@ void alg_quant(celt_norm_t *X, celt_mask_t *W, int N, int K, celt_norm_t *P, ec_
    
    /* Recompute the gain in one pass to reduce the encoder-decoder mismatch
    due to the recursive computation used in quantisation. */
-   mix_pitch_and_residual(iy, X, N, K, P);
+   if (resynth)
+   {
+      normalise_residual(iy, X, N, K, EXTRACT16(SHR32(yy,2*yshift)));
+      if (spread)
+         exp_rotation(X, N, -1, spread, K);
+   }
    RESTORE_STACK;
 }
 
 
 /** Decode pulse vector and combine the result with the pitch vector to produce
     the final normalised signal in the current band. */
-void alg_unquant(celt_norm_t *X, int N, int K, celt_norm_t *P, ec_dec *dec)
+void alg_unquant(celt_norm *X, int N, int K, int spread, celt_norm *lowband, ec_dec *dec)
 {
+   int i;
+   celt_word32 Ryy;
    VARDECL(int, iy);
    SAVE_STACK;
+   if (K==0)
+   {
+      if (lowband != NULL)
+      {
+         for (i=0;i<N;i++)
+            X[i] = lowband[i];
+         renormalise_vector(X, Q15ONE, N, 1);
+      } else {
+         /* This is important for encoding the side in stereo mode */
+         for (i=0;i<N;i++)
+            X[i] = 0;
+      }
+      return;
+   }
+   K = get_pulses(K);
    ALLOC(iy, N, int);
    decode_pulses(iy, N, K, dec);
-   mix_pitch_and_residual(iy, X, N, K, P);
+   Ryy = 0;
+   i=0;
+   do {
+      Ryy = MAC16_16(Ryy, iy[i], iy[i]);
+   } while (++i < N);
+   normalise_residual(iy, X, N, K, Ryy);
+   if (spread)
+      exp_rotation(X, N, -1, spread, K);
    RESTORE_STACK;
 }
 
-void renormalise_vector(celt_norm_t *X, celt_word16_t value, int N, int stride)
+celt_word16 renormalise_vector(celt_norm *X, celt_word16 value, int N, int stride)
 {
    int i;
-   celt_word32_t E = EPSILON;
-   celt_word16_t g;
-   celt_norm_t *xptr = X;
+#ifdef FIXED_POINT
+   int k;
+#endif
+   celt_word32 E = EPSILON;
+   celt_word16 g;
+   celt_word32 t;
+   celt_norm *xptr = X;
    for (i=0;i<N;i++)
    {
       E = MAC16_16(E, *xptr, *xptr);
       xptr += stride;
    }
+#ifdef FIXED_POINT
+   k = celt_ilog2(E)>>1;
+#endif
+   t = VSHR32(E, (k-7)<<1);
+   g = MULT16_16_Q15(value, celt_rsqrt_norm(t));
 
-   g = MULT16_16_Q15(value,celt_rcp(SHL32(celt_sqrt(E),9)));
    xptr = X;
    for (i=0;i<N;i++)
    {
-      *xptr = PSHR32(MULT16_16(g, *xptr),8);
+      *xptr = EXTRACT16(PSHR32(MULT16_16(g, *xptr), k+1));
       xptr += stride;
    }
-}
-
-static void fold(const CELTMode *m, int N, celt_norm_t *Y, celt_norm_t * restrict P, int N0, int B)
-{
-   int j;
-   const int C = CHANNELS(m);
-   int id = N0 % (C*B);
-   /* Here, we assume that id will never be greater than N0, i.e. that 
-      no band is wider than N0. In the unlikely case it happens, we set
-      everything to zero */
-   if (id+C*N>N0)
-      for (j=0;j<C*N;j++)
-         P[j] = 0;
-   else
-      for (j=0;j<C*N;j++)
-         P[j] = Y[id++];
-}
-
-#define KGAIN 6
-
-void intra_fold(const CELTMode *m, celt_norm_t * restrict x, int N, int K, celt_norm_t *Y, celt_norm_t * restrict P, int N0, int B)
-{
-   celt_word16_t pred_gain;
-   const int C = CHANNELS(m);
-
-   if (K==0)
-      pred_gain = Q15ONE;
-   else
-      pred_gain = celt_div((celt_word32_t)MULT16_16(Q15_ONE,N),(celt_word32_t)(N+KGAIN*K));
-
-   fold(m, N, Y, P, N0, B);
-
-   renormalise_vector(P, pred_gain, C*N, 1);
+   return celt_sqrt(E);
 }
 
