@@ -1,4 +1,6 @@
-/* Copyright (C) 2008 Jean-Marc Valin, CSIRO
+/* Copyright (c) 2002-2007 Jean-Marc Valin
+   Copyright (c) 2008 CSIRO
+   Copyright (c) 2007-2009 Xiph.Org Foundation
    File: celtdec.c
 
    Redistribution and use in source and binary forms, with or without
@@ -83,7 +85,7 @@
 #include "wav_io.h"
 #include <celt_header.h>
 
-#define MAX_FRAME_SIZE 2000
+#define MAX_FRAME_SIZE 2048
 
 #define readint(buf, base) (((buf[base+3]<<24)&0xff000000)| \
                            ((buf[base+2]<<16)&0xff0000)| \
@@ -286,7 +288,7 @@ void version_short(void)
    printf ("Copyright (C) 2008 Jean-Marc Valin\n");
 }
 
-static CELTDecoder *process_header(ogg_packet *op, celt_int32_t enh_enabled, celt_int32_t *frame_size, int *granule_frame_size, celt_int32_t *rate, int *nframes, int forceMode, int *channels, int *overlap, int *extra_headers, int quiet, CELTMode **mode)
+static CELTDecoder *process_header(ogg_packet *op, celt_int32 enh_enabled, celt_int32 *frame_size, int *granule_frame_size, celt_int32 *rate, int *nframes, int forceMode, int *channels, int *overlap, int *extra_headers, int quiet, CELTMode **mode)
 {
    CELTDecoder *st;
    CELTHeader header;
@@ -299,7 +301,7 @@ static CELTDecoder *process_header(ogg_packet *op, celt_int32_t enh_enabled, cel
       fprintf (stderr, "Unsupported number of channels: %d\n", header.nb_channels);
       return NULL;
    }
-   *mode = celt_mode_create(header.sample_rate, header.nb_channels, header.frame_size, NULL);
+   *mode = celt_mode_create(header.sample_rate, header.frame_size, NULL);
    if (*mode == NULL)
    {
       fprintf (stderr, "Mode initialization failed.\n");
@@ -313,14 +315,15 @@ static CELTDecoder *process_header(ogg_packet *op, celt_int32_t enh_enabled, cel
    
    *channels = header.nb_channels;
    *overlap=header.overlap;
-   st = celt_decoder_create(*mode);
+   st = celt_decoder_create(*mode, header.nb_channels, NULL);
    if (!st)
    {
       fprintf (stderr, "Decoder initialization failed.\n");
       return NULL;
    }
    
-   celt_mode_info(*mode, CELT_GET_FRAME_SIZE, frame_size);
+   /*celt_mode_info(*mode, CELT_GET_FRAME_SIZE, frame_size);*/
+   *frame_size = header.frame_size;
    *granule_frame_size = *frame_size;
 
    if (!*rate)
@@ -567,9 +570,9 @@ int main(int argc, char **argv)
                   int ret;
                   /*Decode frame*/
                   if (!lost)
-                     ret = celt_decode(st, (unsigned char*)op.packet, op.bytes, output);
+                     ret = celt_decode(st, (unsigned char*)op.packet, op.bytes, output, frame_size);
                   else
-                     ret = celt_decode(st, NULL, 0, output);
+                     ret = celt_decode(st, NULL, 0, output, frame_size);
 
                   /*for (i=0;i<frame_size*channels;i++)
                     printf ("%d\n", (int)output[i]);*/
@@ -581,7 +584,7 @@ int main(int argc, char **argv)
                   }
 
                   if (print_bitrate) {
-                     celt_int32_t tmp=op.bytes;
+                     celt_int32 tmp=op.bytes;
                      char ch=13;
                      fputc (ch, stderr);
                      fprintf (stderr, "Bitrate in use: %d bytes/packet     ", tmp);
