@@ -1,5 +1,6 @@
-/* Copyright (C) 2002-2008 Jean-Marc Valin 
-   Copyright (C) 2008-2009 Gregory Maxwell
+/* Copyright (c) 2002-2007 Jean-Marc Valin 
+   Copyright (c) 2007-2009 Xiph.Org Foundation
+   Copyright (c) 2008-2009 Gregory Maxwell
    File: celtenc.c
 
    Redistribution and use in source and binary forms, with or without
@@ -79,13 +80,13 @@ int oe_write_page(ogg_page *page, FILE *fp)
    return written;
 }
 
-#define MAX_FRAME_SIZE 2000
-#define MAX_FRAME_BYTES 300
+#define MAX_FRAME_SIZE 2048
+#define MAX_FRAME_BYTES 1024
 #define IMIN(a,b) ((a) < (b) ? (a) : (b))   /**< Minimum int value.   */
 #define IMAX(a,b) ((a) > (b) ? (a) : (b))   /**< Maximum int value.   */
 
 /* Convert input audio bits, endians and channels */
-static int read_samples(FILE *fin,int frame_size, int bits, int channels, int lsb, short * input, char *buff, celt_int32_t *size)
+static int read_samples(FILE *fin,int frame_size, int bits, int channels, int lsb, short * input, char *buff, celt_int32 *size)
 {   
    unsigned char in[MAX_FRAME_SIZE*2];
    int i;
@@ -167,7 +168,7 @@ void add_fishead_packet (ogg_stream_state *os) {
 /*
  * Adds the fishead packets in the skeleton output stream along with the e_o_s packet
  */
-void add_fisbone_packet (ogg_stream_state *os, celt_int32_t serialno, CELTHeader *header) {
+void add_fisbone_packet (ogg_stream_state *os, celt_int32 serialno, CELTHeader *header) {
 
    fisbone_packet fp;
 
@@ -188,13 +189,13 @@ void add_fisbone_packet (ogg_stream_state *os, celt_int32_t serialno, CELTHeader
 void version(void)
 {
    printf ("celtenc (CELT %s encoder)\n",CELT_VERSION);
-   printf ("Copyright (C) 2008 Jean-Marc Valin\n");
+   printf ("Copyright (C) 2008-2010 Xiph.Org Foundation (written by Jean-Marc Valin)\n");
 }
 
 void version_short(void)
 {
    printf ("celtenc (CELT %s encoder)\n",CELT_VERSION);
-   printf ("Copyright (C) 2008 Jean-Marc Valin\n");
+   printf ("Copyright (C) 2008-2010 Xiph.Org Foundation (written by Jean-Marc Valin)\n");
 }
 
 void usage(void)
@@ -247,7 +248,7 @@ int main(int argc, char **argv)
    char *inFile, *outFile;
    FILE *fin, *fout;
    short input[MAX_FRAME_SIZE];
-   celt_int32_t frame_size = 256;
+   celt_int32 frame_size = 256;
    int quiet=0;
    int nbBytes;
    CELTMode *mode;
@@ -283,8 +284,8 @@ int main(int argc, char **argv)
       {0, 0, 0, 0}
    };
    int print_bitrate=0;
-   celt_int32_t rate=44100;
-   celt_int32_t size;
+   celt_int32 rate=44100;
+   celt_int32 size;
    int chan=1;
    int fmt=16;
    int lsb=1;
@@ -303,7 +304,7 @@ int main(int argc, char **argv)
    float bitrate=-1;
    char first_bytes[12];
    int wave_input=0;
-   celt_int32_t lookahead = 0;
+   celt_int32 lookahead = 0;
    int bytes_per_packet=48;
    int complexity=-127;
    int prediction=2; 
@@ -490,7 +491,7 @@ int main(int argc, char **argv)
      bitrate = ((rate/(float)frame_size)*8*bytes_per_packet)/1000.0;
    }
    
-   mode = celt_mode_create(rate, chan, frame_size, NULL);
+   mode = celt_mode_create(rate, frame_size, NULL);
    if (!mode)
       return 1;
 
@@ -499,9 +500,9 @@ int main(int argc, char **argv)
    snprintf(vendor_string, sizeof(vendor_string), "Encoded with CELT %s (bitstream: %d)\n",CELT_VERSION,bitstream);
    comment_init(&comments, &comments_length, vendor_string);
 
-   celt_mode_info(mode, CELT_GET_FRAME_SIZE, &frame_size);   
+   /*celt_mode_info(mode, CELT_GET_FRAME_SIZE, &frame_size);*/
    
-   celt_header_init(&header, mode);
+   celt_header_init(&header, mode, chan);
    header.nb_channels = chan;
 
    {
@@ -518,7 +519,7 @@ int main(int argc, char **argv)
    }
 
    /*Initialize CELT encoder*/
-   st = celt_encoder_create(mode);
+   st = celt_encoder_create(mode, chan, NULL);
 
    if (with_vbr)
    {
@@ -663,7 +664,7 @@ int main(int argc, char **argv)
       id++;
       /*Encode current frame*/
 
-      nbBytes = celt_encode(st, input, NULL, bits, bytes_per_packet);
+      nbBytes = celt_encode(st, input, frame_size, bits, bytes_per_packet);
       if (nbBytes<0)
       {
          fprintf(stderr, "Got error %d while encoding. Aborting.\n", nbBytes);
